   function [B, XN, Pr, wo] = coarsestep3(D, X, wx)
%  function [B, XN, Pr, wo] = coarsestep3(D, X, wx)
% Algorithm proceeds by traversing VERTICES - based on wx
%%    (largest to smallest weights in wx)     
%
% IN
%  D   = weight matrix..  
%  X   = data matrix - each columns contains a point (e.g, xyz
%        coordinates or some other features);
%  wx  = weights of vertices [row vector] [1,n] 
%
% OUT: 
%  B    = Output sparse  matrix of weights - may be used for next level
%  XN   = coarse nodes (coarsened `features')
%        in coarse graph 
% wo    = weights for output nodes  [row vector]
%  Pr   = Projection matrix used to produce B, wo,  and XN:
%          [B = Pr'*D*Pr;  XN = X*Pr;  wo = wx*Pr; ]
%% ASSUMES DIAGONAL OF D IS ZERO.

N = size(D,1);     % Numbers of nodes of the given (fine) graph.
prnt = zeros(N,1); % parents list 
%%-------------------- unroll sparse matrix into arrays.
%%[I, J, W] = find(D);
new = 0;
[~, indx] = sort(wx,'descend');
n = size(D,1);
kp = 0;
for kj = 1:n
  j = indx(kj);
  if (prnt(j) >0), continue, end
%%  next = indx(k);
  [jr, jc, wj] = find(D(:,j));
  [~, idx] = sort(wj,'descend');
%%-------------------- select best unmarked candidate
  mat = 0;
  len = length(idx);
  for k=1:len
    ji = jr(k);
%%-------------------- if already visited or if diag. entry skip
    if (prnt(ji) == 0 & ji ~= j) 
      mat = jr(k);
      break;
    end
  end
%%-------------------- if found match create new coarse node
  if (mat >0)
    new=new+1;
    prnt(j) = new;
    prnt(mat) = new;
%%-------------------- update projection matrix in COO format            
%%  1st
    kp = kp+1;
    ip(kp) = j;
    jp(kp) = new;
%% 2nd
    kp = kp+1;
    ip(kp) = mat;
    jp(kp) = new;
  elseif(len)
    mat = jr(1);
    prnt(j) = prnt(mat);
  else
%%-------------------- add True singleton
    new=new+1;
    prnt(j) = new;
    kp = kp+1;
    ip(kp) = j;
    jp(kp) = new;
  end
end
%%-------------------- now build the projection graph;
Pr = sparse(ip, jp, 1, N, new);
for j=1:new
  [ir, ~, ~] = find(Pr(:,j));
  t = norm(wx(ir));
  Pr(ir,j)  = wx(ir)'/t;
end
%%-------------------- reorder Pr to best preserve original structure 
for j=1:new
  ir(j) = min(find(Pr(:,j)));
end
[~, idx] = sort(ir);
Pr = Pr(:,idx);
%%-------------------- return modified objects
XN = X*Pr;
wo = wx*Pr;
B = Pr'*D*Pr;
